%%
%============================= VERSION 1 =================================%
%- SF6Concentration.m (Matlab routine)                                   -%
%-                                                                       -%
%- Routine to plot the SF6 concentration from an ASCII (or .txt) file    -%
%- which contains (minimum) SF6 concentration / date                     -%
%-                                                                       -%
%- Execution: submit "SF6Concentration_vx" in a Matlab command window    -%
%-                                                                       -%
%- Files needed:                                                         -%
%-      - Iridium buoy data (x files)                                    -%
%-      - Vessel Suroit position data (pos_bateau.txt)                   -%
%-      - SF6 concentration data (sf6_mapping.txt)                       -%
%-                                                                       -%
%- Output :                                                              -%
%-      - 1 file with:                                                   -%
%-          SF6-date SF6-Lon SF6-Lat SF6-Conc. Iridium-Lon Iridium-Lat   -%
%-      - 4 figures: vessel suroit, Iridium buoy, SF6 concentration patch-%
%-         - geographic plot vessel, SF6 conc. patch, Iridium buoy       -%
%-         - lagrangian plot vessel, SF6 conc. patch                     -%
%-         - geographic 2D mapping of the SF6 concentration (interp)     -%
%-         - lagrangian 2D mapping of the SF6 concentration (interp)     -%
%-                                                                       -%
%- Matlab library needed:                                                -%
%-      - m_map                                                          -%
%=========================================================================%
%- 08-13 September 2010                                                  -%
%- Nicolas.Grima@suroit.ifremer.fr du 6 au 24 septembre 2010             -%
%- Nicolas.Grima@univ-brest.fr                                           -%
%- Merci  "Rose Campbel" pour sa participation.                         -%
%-------------------------------------------------------------------------%

%%
%=============%
%- Algorythm -%
%=============%-----------------------------------------------------------%
%- READ Iridium buoy coordinates during the same period of the SF6       -%
%- concentration measurements.                                           -%
%-                                                                       -%
%- READ the vessel position data which contains the longitudes, latitudes-%
%-                                                                       -%
%- READ the ASCII file which contains the SF6 date and SF6 concentration.-%
%- (remark: the lon/lat coordinates of the SF6 correspond to the boat    -%
%-  coordinates)                                                         -%
%-                                                                       -%
%- CALCULATE the position of the SF6 concentration using the buoy iridium-% 
%- as the center of the axes.                                            -%
%-                                                                       -%
%- PLOT figures                                                          -%
%-                                                                       -%
%- SAVE figures.                                                         -%
%-------------------------------------------------------------------------%

clear all; close all;

%%
%==============%
%= Paramaters =%
%==============%------------------------------------------%
%- Feel free here to change the value of some parameters -%
%---------------------------------------------------------%
ScanEtrackDir = 'ScanEtrack'; % ScanEtrack directory with pos_bateau.txt 
                              % and trame directory.
                              
SF6SeuilHaut = 280000;            
SF6SeuilBas  = 3500;  

SF6PatchSize = 10; % SF6 patch size on figures

SF6Map2DBin = 0.01;

%- Geographic represention -%
FigMinLon = 3.40; % Longitude minimun
FigMaxLon = 3.70; % Longitude maximum

FigMinLat = 42.65; % Latitude minimum
FigMaxLat = 42.85; % Latitude maximum

projection = 'mercator'; % m_map projection

%%
%====================================================================%
%= Enter the Iridium directory where buoy data and vessel should be =%
%====================================================================%
%- Dialog box to select the buoys data directory "iridium" -%
%IridiumDir  = uigetdir('.','Select the directory "Iridium"');
IridiumDir  = 'D:\ANDREA\RECHERCHE\LATEX\latex10\Iridium';

%- Comments -%
disp(' ');
disp(['The Iridium directoyr is: ', IridiumDir]);

%%
%==========================%
%= READ Iridium buoy data =%
%==========================%----------------------------------------------%
%- A dialog box will be opened to select the buoy file directory         -%
%- Then buoy files corresponding to the SF6 data period will be selected -%
%- and data read.                                                        -%
%-------------------------------------------------------------------------%
%- Comments -%
disp(' ');
disp('Read Buoy Data files');

%- All the buoys file names + directory found in BuoyFilesDir -%
%BuoyFilesDir = fullfile([IridiumDir,'\',ScanEtrackDir,'\trame'])
BuoyFilesDir = 'D:\ANDREA\RECHERCHE\LATEX\latex10\Iridium\ScanEtrack\trame';

disp(BuoyFilesDir);
BuoyFiles    = dir(fullfile(BuoyFilesDir,'*.txt'));

%- Only buoy file names (whithout the path) -%
BuoyFilesName = {BuoyFiles.name};

%- Transform the character name in Integer -%
BuoyFilesNameInt = char(BuoyFilesName);
%- Extract the date form the file name -%
BuoyFilesDate    = BuoyFilesNameInt(:,1:14);
%- Calculate the number of buoy files -%
[isz,jsz]        = size(BuoyFilesDate);
Col_T(1:isz)     = 'T';
%- Extract the date in "julian" day to make comparison bellow -%
BuoyDate=datenum(                                ...
    [BuoyFilesDate(:,1:8),Col_T',BuoyFilesDate(:,9:14)], ...
    'yyyymmddTHHMMSS');

% %- Select the buoy files corresponding to the period of SF6 concentration
% %- data -%
% BuoyFilesInd     = ...
%     find((BuoyDate >= SF6MinDate) & (BuoyDate <= SF6MaxDate));
% 
% if isempty(BuoyFilesInd)
%     disp(' ');
%     disp(['There is no common period between '...
%           'SF6 concentration and Buoy data']);
%     break;
% end
% 
% BuoyFilesLength  = length(BuoyFilesInd);

BuoyNb  = length(BuoyDate);

%- Initialise some array (performance) -%
BuoyLat  = zeros(BuoyNb,1);
BuoyLon  = zeros(BuoyNb,1);
BuoyTime = zeros(BuoyNb,1);

%- Loop on the selected buoy Files and read buoy data -%
for ii = 1: BuoyNb
  BuoyFilesNameSelected = BuoyFiles(ii).name;
  disp(BuoyFilesNameSelected);
  FileId=fopen(fullfile(BuoyFilesDir,BuoyFilesNameSelected),'r');
  trame=fread(FileId,'*char')';
  comma=strfind(trame,';');
  BuoyLat(ii)  = str2double(trame(comma(1)+1:comma(2)-1));
  BuoyLon(ii)  = str2double(trame(comma(2)+1:comma(3)-1));
  BuoyTimeTmp  = datevec(trame(1:19),'yyyy-mm-dd HH:MM:SS');
  % BuoyTime(ii) = datenum(BuoyTimeTmp)* 86400.; % in seconds
  BuoyTime(ii) = datenum(BuoyTimeTmp);
  fclose(FileId);
end

%- Calculate the min and the max of the date -%
BuoyMinDate = min(BuoyTime);
BuoyMaxDate = max(BuoyTime);

disp(['Buoy data number: ',num2str(BuoyNb)]);
disp('Buoy min and max date:');
disp(['    -', num2str(BuoyMinDate)]);
disp(['    -', num2str(BuoyMaxDate)]);


%%
%=============================%
%= READ vessel position data =%
%=============================%
%- Comments -%
disp(' ');
disp('Read Vessel Suroit Data file (pos_bateau.txt)');

% flag = 1;
% VesselPathName='.';
% %- While loop to select a valid vessel data file -%
% while flag
%     %- Dialog box which asks you to select a *.txt file -%
%     [VesselFileName,VesselPathName] = ...
%         uigetfile('*.txt', ...
%         'Select Vessel position file (pos_bateau.txt)', ...
%         VesselPathName);
%     %- Test if the file selected is valid -%
%     if isequal(VesselFileName,0)
%         disp('Finish the program')
%         flag = 1;
%         break;
%     else
%         if strcmp(VesselFileName, 'pos_bateau.txt')
%           disp(['File selected: ', ...
%               fullfile(VesselPathName, VesselFileName)])
%           flag = 0;
%         else
%             disp('Please select a file named "pos_bateau.txt"');
%             flag = 1;
%         end
%     end
% end

FileId = fopen(fullfile(IridiumDir,ScanEtrackDir,'pos_bateau.txt'));
disp(fullfile(IridiumDir,ScanEtrackDir,'pos_bateau.txt'));

Trames = fread(FileId,'*char')';

fclose(FileId);

TramesIndINGGA = strfind(Trames,'$INGGA');
TramesIndINRMC = strfind(Trames,'$INRMC');

TramesLength = length(TramesIndINGGA);

VesselLatAll   = zeros(TramesLength-1,1);
VesselLonAll   = zeros(TramesLength-1,1);
VesselTimeAll  = zeros(TramesLength-1,1);
VesselCapeAll  = zeros(TramesLength-1,1);
VesselSpeedAll = zeros(TramesLength-1,1);

disp('');
disp('Please wait....!');

for ii = 1: TramesLength-1
  TrameINGGA = Trames(TramesIndINGGA(ii):TramesIndINGGA(ii+1));
  comma=strfind(TrameINGGA,',');
  VesselLatTmp  = str2double(TrameINGGA(comma(2)+1:comma(3)-1))/100.;
  VesselLatAll(ii) = conversion(VesselLatTmp); %
  VesselLonTmp  = str2double(TrameINGGA(comma(4)+1:comma(5)-1))/100;
  VesselLonAll(ii) = conversion(VesselLonTmp);
  VesselHour    = (TrameINGGA(comma(1)+1:comma(2)-4));
  
  TrameINRMC = Trames(TramesIndINRMC(ii):TramesIndINRMC(ii+1));
  comma=strfind(TrameINRMC,',');
  VesselDate     = (TrameINRMC(comma(9)+1:comma(10)-1));
  %- time in seconds -%
  VesselTimeAll(ii) = datenum([VesselDate,VesselHour],'ddmmyyHHMMSS');
  VesselCapeAll(ii) = str2double(TrameINRMC(comma(8)+1:comma(9)-1));
  VesselSpeedAll(ii) = str2double(TrameINRMC(comma(7)+1:comma(8)-1));
end

clear Trames TramesIndINGGA TramesIndINRMC TrameINGGA TrameINRMC; 

VesselTimeInd = ...
    find((VesselTimeAll >= BuoyMinDate) & (VesselTimeAll <= BuoyMaxDate));

if isempty(VesselTimeInd)
    disp(' ');
    disp(['There is no common period between '...
          'Buoy and vessel data']);
    break;
end

VesselNb    = length(VesselTimeInd);
VesselLat   = VesselLatAll(VesselTimeInd);
VesselLon   = VesselLonAll(VesselTimeInd);
VesselTime  = VesselTimeAll(VesselTimeInd);
VesselCape  = VesselCapeAll(VesselTimeInd);
VesselSpeed = VesselSpeedAll(VesselTimeInd);

clear VesselLatAll VesselLonAll VesselTimeAll VesselCapeAll VesselSpeedAll;

%- Calculate the min and the max of the date -%
VesselMinDate = min(VesselTime);
VesselMaxDate = max(VesselTime);

disp(['Vessel data number: ',num2str(VesselNb)]);
disp('Vessel min and max date:');
disp(['    -', num2str(VesselMinDate)]);
disp(['    -', num2str(VesselMaxDate)]);


%%
%===============================%
%= READ SF6 Concentration data =%
%===============================%
%- use the matlab routine load to read the SF6 concentration
%- S = load(filename, '-ascii')
%- uigetfile to select file (but uiimport could be used !)
%----------------------------------
% flag = 1;
% %- While loop to select a valid SF6 concentration file -%
% while flag
%     %- Dialog box which asks you to select a *.txt file -%
%     [SF6FileName,SF6PathName] = ...
%         uigetfile('*.txt','Select SF6 concentration file');
%     %- Test if the file selected is valid -%
%     if isequal(SF6FileName,0)
%         disp('User selected Cancel')
%         flag = 1;
%     else
%         disp(['User selected', fullfile(SF6PathName, SF6FileName)])
%         flag = 0;
%     end
% end
% 
% %- Assuming that the data in the SF6concentration ascii file 
% %- are store as: longitude, latitude, concentration, date
% %- Can be changed or adapted
% vars = {'SF6Lon', 'SF6Lat', 'SF6Conc', 'SF6date'};
% 
% %- Load data -%
% load(fullfile(SF6PathName, SF6FileName),'-ascii', vars{:});
disp(' ');
disp('Read SF6 data.');
disp('Please select the SF6 data file "sf6_mapping.txt"');

newData1=uiimport(fullfile(IridiumDir,'sf6_mapping.txt'));

vars = fieldnames(newData1);

for i = 1:length(vars)
    assignin('base', vars{i}, newData1.(vars{i}));
end

SF6ConcAll = data(:,5);

SF6ConcValidInd = find(~isnan(SF6ConcAll));

SF6LonAll  = data(SF6ConcValidInd,1);
SF6LatAll  = data(SF6ConcValidInd,2);
SF6ConcAll = data(SF6ConcValidInd,5);

SF6ConcValidIndLength = length(SF6ConcValidInd);

SF6TimeAll = zeros(SF6ConcValidIndLength,1);

for ii = 1: SF6ConcValidIndLength
  SF6TimeAll(ii) = datenum([textdata{ii+1,4},textdata{ii+1,5}], ...
                         'mm/dd/yyyyHH:MM');
end

SF6TimeInd = ...
    find((SF6TimeAll >= BuoyMinDate) & (SF6TimeAll <= BuoyMaxDate));

if isempty(SF6TimeInd)
    disp(' ');
    disp(['There is no common period between '...
          'Buoy and SF6 data']);
    break;
end

SF6Nb    = length(SF6TimeInd);

SF6Lat   = SF6LatAll(SF6TimeInd);
SF6Lon   = SF6LonAll(SF6TimeInd);
SF6Time  = SF6TimeAll(SF6TimeInd);
SF6Conc  = SF6ConcAll(SF6TimeInd);

clear SF6ConcAll SF6TimeAll SF6LonAll SF6LatAll SF6ConcAll;

%- Calculate Min and Max concentration
SF6MinConc = min(SF6Conc(find(SF6Conc >= SF6SeuilBas)));
SF6MaxConc = max(SF6Conc(:));

%- Calculate the min and the max of the date -%
SF6MinDate = min(SF6Time(:));
SF6MaxDate = max(SF6Time(:));

disp(['SF6 data number: ',num2str(SF6Nb)]);
disp('SF6 min and max date:');
disp(['    -', num2str(SF6MinDate)]);
disp(['    -', num2str(SF6MaxDate)]);

%%
%=========================================================================%
%- Search the coordinates of the SF6 concentration using Vessel positions-%
%=========================================================================%
SF6LatVessel = zeros(SF6Nb,1);
SF6LonVessel = zeros(SF6Nb,1);
VesselSF6Ind = zeros(SF6Nb,1);

disp(' ');

for isf6 = 1: SF6Nb
    
  VesselSF6Ind(isf6) = find((SF6Time(isf6) <= VesselTime) & ...
      (SF6Time(isf6) >= VesselTime));
  
  disp(['SF6Time: ', num2str(SF6Time(isf6)),...
      ' - VesselTime: ', num2str(VesselTime(VesselSF6Ind(isf6))), ...
      ' - VesselSF6Ind: ', num2str(VesselSF6Ind(isf6))]);
  
  SF6LatVessel(isf6) = VesselLat(VesselSF6Ind(isf6));
  SF6LonVessel(isf6) = VesselLon(VesselSF6Ind(isf6));
    
end

%%
%========================================================================%
%= CALCULATE the Vessel and SF6 Concentration positions in the          =%
%= lagrangian reference system of the iridium buoy                      =%
%========================================================================%
%- take idea in : reconstitution.m in LATEX_Tools directory
%- key routine  : m_idist and pol2cart
%-----------------------------------------------------
%-new array repeting same position of buoy -%
BuoyDeltaTNew = (BuoyMaxDate - BuoyMinDate) / (VesselNb - 1);
BuoyTimeNew   = BuoyMinDate:BuoyDeltaTNew:BuoyMaxDate;
BuoyLonNew = interp1(BuoyTime, BuoyLon, BuoyTimeNew,'linear')';
BuoyLatNew = interp1(BuoyTime, BuoyLat, BuoyTimeNew,'linear')';

%distance calculation betweeb vessel and buoy
[ss,a12,a21] = m_idist(BuoyLonNew,BuoyLatNew,VesselLon,VesselLat);
theta=(90.-a12)*pi/180.;
[distx,disty]=pol2cart(theta,ss);%[m]
distx=distx./1000.; %[km]
disty=disty./1000.; %[km]

%%
%==========================================================%
%- Save SF6 data in the file SF6_mapping_vessel_coord.txt -%
%==========================================================%
FileId = fopen(fullfile(IridiumDir,...
                        'sf6_mapping_vessel_coord.txt'),'w');
fprintf(FileId, ...
'SF6-Date SF6-Time SF6-Suroit-Lon SF6-Suroit-Lat SF6-Concent.  Iridium-Lon  Iridium-Lat\n');
for ii= 1:SF6Nb
 fprintf(FileId,'%10s %5s %12.8f %12.8f %12.8f %12.8f %12.8f\n',...
     textdata{ii+1,4}             , ...
     textdata{ii+1,5}             , ...
     SF6LonVessel(ii)             , ...
     SF6LatVessel(ii)             , ...
     SF6Conc(ii)                  , ...
     BuoyLonNew(VesselSF6Ind(isf6)), ...
     BuoyLatNew(VesselSF6Ind(isf6)));
end

fclose(FileId);

%%
% Interpolate SF6 to 2D grid
F= TriScatteredInterp(SF6LonVessel,SF6LatVessel,SF6Conc);
[longi, lati]=meshgrid(FigMinLon:SF6Map2DBin:FigMaxLon,...
    FigMinLat:SF6Map2DBin:FigMaxLat);
SF6interp=F(longi,lati);

%%
% Interpolate SF6 to 2D grid in lagrangian repere
FL= TriScatteredInterp(distx(VesselSF6Ind(:)),...
    disty(VesselSF6Ind(:)),SF6Conc);
[distxi, distyi]=meshgrid(min(distx):0.5:max(distx),...
    min(disty):0.5:max(disty));
SF6interpL=FL(distxi,distyi);

%%
%================%
%= Plot figures =%
%================%
FigId1 = figure(1);
m_proj(projection,...
    'lon',[FigMinLon FigMaxLon], ...
    'lat',[FigMinLat FigMaxLat]);

hold on;
m_usercoast('GoL','patch',[.7 .7 .7])

hold on;
m_line(VesselLon,VesselLat,'color','b');
% m_plot(VesselLon,VesselLat,'.b');

hold on;
m_line(BuoyLon,BuoyLat,'color','r');
m_plot(BuoyLon,BuoyLat,'.r');

hold on;
cmap = colormap(jet);
szcmap = size(cmap,1);

SF6ConcDelta = (SF6SeuilHaut - SF6SeuilBas) / (szcmap - 1);

for ii= 1:SF6Nb
  if (SF6Conc(ii) >= SF6SeuilBas && SF6Conc(ii) < SF6SeuilHaut)
    SF6ConcColor = int32((SF6Conc(ii) - SF6MinConc)./ SF6ConcDelta) + 1;
    m_plot(SF6LonVessel(ii),SF6LatVessel(ii),'s',...
      'MarkerEdgeColor',cmap(SF6ConcColor,:),...
      'MarkerFaceColor',cmap(SF6ConcColor,:),...
      'MarkerSize',SF6PatchSize);
  elseif(SF6Conc(ii) >= SF6SeuilHaut)
    SF6ConcColor = int32((SF6SeuilHaut - SF6MinConc)./ SF6ConcDelta) + 1;
    m_plot(SF6LonVessel(ii),SF6LatVessel(ii),'s',...
      'MarkerEdgeColor',cmap(SF6ConcColor,:),...
      'MarkerFaceColor',cmap(SF6ConcColor,:),...
      'MarkerSize',SF6PatchSize);
  else
    m_plot(SF6LonVessel(ii),SF6LatVessel(ii),'s',...
      'MarkerEdgeColor','k',...
      'MarkerFaceColor','w',...
      'MarkerSize',SF6PatchSize);
  end
end

colorbar;
caxis([SF6SeuilBas SF6SeuilHaut]);

Plot_buoy('M:\LATEX\Bouee\',...
    FigMinLon,FigMinLat,FigMaxLon,FigMaxLat);

m_grid;
title('Vessel (blue) - Buoy (red) tracks and SF6 Concentration')
xlabel('longitude');
ylabel('latitude');

orient landscape;
print('-dtiff',[IridiumDir,'\Buoy_Vessel_SF6_mapping.tif']);
%%
%= in Buoy Lagrangian repere system =%
FigId2 = figure(2);

hold on;
plot(distx,disty,'-b');

hold on;
plot(0,0,'or');

hold on;
for ii= 1:SF6Nb
  if (SF6Conc(ii) >= SF6SeuilBas && SF6Conc(ii) < SF6SeuilHaut)
    SF6ConcColor = int32((SF6Conc(ii) - SF6MinConc)  ./ SF6ConcDelta) + 1;
    plot(distx(VesselSF6Ind(ii)),disty(VesselSF6Ind(ii)),'s',...
      'MarkerEdgeColor',cmap(SF6ConcColor,:),...
      'MarkerFaceColor',cmap(SF6ConcColor,:),...
      'MarkerSize',SF6PatchSize);
  elseif(SF6Conc(ii) >= SF6SeuilHaut)
    SF6ConcColor = int32((SF6SeuilHaut - SF6MinConc)./ SF6ConcDelta) + 1;
    plot(distx(VesselSF6Ind(ii)),disty(VesselSF6Ind(ii)),'s',...
      'MarkerEdgeColor',cmap(SF6ConcColor,:),...
      'MarkerFaceColor',cmap(SF6ConcColor,:),...
      'MarkerSize',SF6PatchSize);
  else
    plot(distx(VesselSF6Ind(ii)),disty(VesselSF6Ind(ii)),'s',...
      'MarkerEdgeColor','k',...
      'MarkerFaceColor','w',...
      'MarkerSize',SF6PatchSize);
  end
end

colorbar;
caxis([SF6SeuilBas SF6SeuilHaut]);

title('vessel track and SF6 concentration in lagrangian reference');
xlabel('Distance in km');
ylabel('Distance in km');
grid on;

orient landscape;
print('-dtiff',[IridiumDir,'\Buoy_Vessel_SF6_lagrangian_mapping.tif']);

%% 2D pcolor of SF6
FigId3 = figure(3);

m_proj(projection,...
    'lon',[FigMinLon FigMaxLon], ...
    'lat',[FigMinLat FigMaxLat]);

m_pcolor(longi,lati,SF6interp);

colorbar;
caxis([SF6SeuilBas SF6SeuilHaut]);
shading flat;

hold on;
for ii= 1:SF6Nb
  if (SF6Conc(ii) >= SF6SeuilBas)
    SF6ConcColor = int32((SF6Conc(ii) - SF6MinConc)./ SF6ConcDelta) + 1;
    m_plot(SF6LonVessel(ii),SF6LatVessel(ii),'ow',...
      'MarkerSize',6);
  end
end

title('SF6 Concentration 2D map')
xlabel('longitude');
ylabel('latitude');

m_usercoast('GoL','patch',[.7 .7 .7]);
m_grid

orient landscape;
print('-dtiff',[IridiumDir,'\Buoy_Vessel_SF6_2Dmapping.tif']);

%% 2D pcolor of SF6 in lagrangian repere
FigId4 = figure(4);

pcolor(distxi,distyi,SF6interpL);

colorbar;
caxis([SF6SeuilBas SF6SeuilHaut]);
shading flat;

hold on;
for ii= 1:SF6Nb
  if (SF6Conc(ii) >= SF6SeuilBas)
    SF6ConcColor = int32((SF6Conc(ii) - SF6MinConc)./ SF6ConcDelta) + 1;
    plot(distx(VesselSF6Ind(ii)),disty(VesselSF6Ind(ii)),'ow',...
      'MarkerSize',6);
  end
end

hold on;
plot(0,0,'or','MarkerFaceColor','r');

title('SF6 Concentration 2D map in lagrangian reference')
xlabel('distance in km');
ylabel('distance in km');

orient landscape;
print('-dtiff',[IridiumDir,'\Buoy_Vessel_SF6_lagrangian_2Dmapping.tif']);

%%
%================%
%= SAVE figures =%
%================%